#!/bin/bash
#
# Copyright (C) 2019 Gigaset Communications GmbH
#

source /usr/sbin/gigaset-log-functions

supervisor_timeout=30

CCPATH="/data/certs/client/"					# Client certificate path
CACRT="${CCPATH}GigasetProCA2_cert.pem"			# Intermediate certificate authority
CAKEY="${CCPATH}GigasetProCA2_key.pem"			# Intermediate certificate authority key
CAPASS="${CCPATH}GigasetProCA2_pass.txt"		# Intermediate certificate authority password
NEWKEY="${CCPATH}gigaset_maxwell_key.pem"		# New client private key
NEWCRT="${CCPATH}gigaset_maxwell_cert.pem"		# New client certificate
NEWCSR="${CCPATH}gigaset_maxwell_csr.pem"		# New client certificate signing request
FACTORYKEY="${CCPATH}gigaset_factory_key.pem"	# Factory client private key
FACTORYCRT="${CCPATH}gigaset_factory_cert.pem"	# Factory client certificate

# Verify client certificate
function verifyCertificate()
{
	local ca=${1}
	local crt=${2}

	test ! -f ${ca} && { log_error "File ${ca} not found"; return 1; }
	test ! -f ${crt} && { log_error "File ${crt} not found"; return 1; }

	local retval=$(openssl verify -purpose sslclient -CAfile "${ca}" "${crt}")

	log_info "Verify client certificate: ${retval} ---"

	test "${retval##* }" == "OK"
}

# Verify private key
function verifyKey
{
	local key=${1}
	local crt=${2}

	test ! -f ${key} && { log_error "File ${key} not found"; return 1; }
	test ! -f ${crt} && { log_error "File ${crt} not found"; return 1; }

	local retval=$(openssl rsa -check -noout -in "${key}")

	log_info "Verify private key: ${retval} ---"

	test "${retval##* }" != "ok" && return 1

	local moduluskey=$(openssl rsa -noout -modulus -in "${key}")
	local moduluscrt=$(openssl x509 -noout -modulus -in "${crt}")

	test -n "${moduluskey}" && test -n "${moduluscrt}" && test "${moduluskey}" == "${moduluscrt}"
}

# Generate new certificate
function generateCertificate()
{
	# Days to 2028-11-11
	local days="$((($(date +%s --date "2028-11-11")-$(date +%s))/(3600*24)))"

	# Serial number
	local snum="$(openssl rand -hex 8)"

	# Ethernet hardware address
	local ethaddr="$(fw_printenv ethaddr | cut -d '=' -f 2)"

	# Client certificate subject
	local subj="/C=DE/ST=Germany/L=Bocholt/O=Gigaset Communications GmbH/CN=${ethaddr//:}"

	log_info "Current time: `date`"

	log_info "Create the client key"
	! openssl genrsa -out "${NEWKEY}.tmp" 2048 && { log_error "Create the client key failed with code $?"; return 1; }

	log_info "Create the client csr"
	! openssl req -new -key "${NEWKEY}.tmp" -out "${NEWCSR}.tmp" -subj "${subj}" && { log_error "Create the client csr failed with code $?"; return 1; }

	log_info "Sign the client certificate with intermediate CA cert"
	! openssl x509 -req -in "${NEWCSR}.tmp" -CA "${CACRT}" -CAkey "${CAKEY}" -out "${NEWCRT}.tmp" -days ${days} -set_serial "0x${snum}" -passin "file:${CAPASS}" && { log_error "Sign the client certificate with intermediate CA cert failed with code $?"; return 1; }

	log_info "Verify new certificate"
	! verifyCertificate "${CACRT}" "${NEWCRT}.tmp" && { log_error "Verify new certificate failed"; return 1; }

	log_info "Verify new key"
	! verifyKey "${NEWKEY}.tmp" "${NEWCRT}.tmp" && { log_error "Verify new key failed"; return 1; }

	test -f "${NEWCRT}" && chattr -i "${NEWCRT}"
	test -f "${NEWKEY}" && chattr -i "${NEWKEY}"

	log_info "Decode encoded SSL certificate"
	! openssl x509 -in "${NEWCRT}.tmp" -text > "${NEWCRT}" && { log_error "Decode encoded SSL certificate failed with code $?"; return 1; }

	log_info "Commit private key"
	mv "${NEWKEY}.tmp" "${NEWKEY}"

	chattr +i "${NEWCRT}"
	chattr +i "${NEWKEY}"
}

if [ "$1" != "start" ]; then
	exit 0
fi

if test -f "${FACTORYKEY}" && test -f "${FACTORYCRT}"; then
	log_info "Found factory certificate and key"
else
	if test -f "${CACRT}" && test -f "${CAKEY}" && test -f "${CAPASS}"; then
		if test -f "${NEWKEY}" && test -f "${NEWCRT}" && verifyCertificate "${CACRT}" "${NEWCRT}" && verifyKey "${NEWKEY}" "${NEWCRT}"; then
			log_info "Certificate and key are correct"
		else
			generateCertificate && log_info "Generate certificate and key succeed" || { log_error "Generate certificate and key failed"; exit 2; }
		fi
	fi
fi

# Remove intermediate CA and all temporary files
rm -f "${CACRT}" "${CAKEY}" "${CAPASS}" "${NEWCRT}.tmp" "${NEWKEY}.tmp" "${NEWCSR}.tmp"
